commit 3fea4d5988de365503df44a9b067e3b181cac65c
Author: Timo Sirainen <timo.sirainen@dovecot.fi>
Date:   Thu Jun 30 14:08:24 2016 +0300

    master: Stopping didn't close dead-pipes early enough.
    
    This caused stopping to be too slow. Broken by 0153cf542.

diff --git a/src/master/service-monitor.c b/src/master/service-monitor.c
index 19655c2..44cdb26 100644
--- a/src/master/service-monitor.c
+++ b/src/master/service-monitor.c
@@ -505,6 +505,14 @@ void services_monitor_start(struct service_list *service_list)
 	}
 }
 
+static void service_monitor_close_dead_pipe(struct service *service)
+{
+	if (service->master_dead_pipe_fd[0] != -1) {
+		i_close_fd(&service->master_dead_pipe_fd[0]);
+		i_close_fd(&service->master_dead_pipe_fd[1]);
+	}
+}
+
 void service_monitor_stop(struct service *service)
 {
 	int i;
@@ -522,10 +530,7 @@ void service_monitor_stop(struct service *service)
 			service->status_fd[i] = -1;
 		}
 	}
-	if (service->master_dead_pipe_fd[0] != -1) {
-		i_close_fd(&service->master_dead_pipe_fd[0]);
-		i_close_fd(&service->master_dead_pipe_fd[1]);
-	}
+	service_monitor_close_dead_pipe(service);
 	if (service->login_notify_fd != -1) {
 		if (close(service->login_notify_fd) < 0) {
 			service_error(service,
@@ -582,6 +587,9 @@ void services_monitor_stop(struct service_list *service_list, bool wait)
 {
 	struct service *const *services;
 
+	array_foreach(&service_list->services, services)
+		service_monitor_close_dead_pipe(*services);
+
 	if (wait) {
 		/* we've notified all children that the master is dead.
 		   now wait for the children to either die or to tell that
commit fde7b8a03bf91cfa5bb7ca3e84545386243fa0d2
Author: Timo Sirainen <timo.sirainen@dovecot.fi>
Date:   Tue Jun 28 23:38:11 2016 +0300

    master: Added support for stopping specific services.
    
    We need to have a per-service fd for detecting a dead master.

diff --git a/src/master/service-monitor.c b/src/master/service-monitor.c
index bf4e931..fb4f64f 100644
--- a/src/master/service-monitor.c
+++ b/src/master/service-monitor.c
@@ -452,11 +452,6 @@ void services_monitor_start(struct service_list *service_list)
 		return;
 	service_anvil_monitor_start(service_list);
 
-	if (pipe(service_list->master_dead_pipe_fd) < 0)
-		i_error("pipe() failed: %m");
-	fd_close_on_exec(service_list->master_dead_pipe_fd[0], TRUE);
-	fd_close_on_exec(service_list->master_dead_pipe_fd[1], TRUE);
-
 	array_foreach(&service_list->services, services) {
 		struct service *service = *services;
 
@@ -464,6 +459,14 @@ void services_monitor_start(struct service_list *service_list)
 			if (service_login_create_notify_fd(service) < 0)
 				continue;
 		}
+		if (service->master_dead_pipe_fd[0] == -1) {
+			if (pipe(service->master_dead_pipe_fd) < 0) {
+				service_error(service, "pipe() failed: %m");
+				continue;
+			}
+			fd_close_on_exec(service->master_dead_pipe_fd[0], TRUE);
+			fd_close_on_exec(service->master_dead_pipe_fd[1], TRUE);
+		}
 		if (service->status_fd[0] == -1) {
 			/* we haven't yet created status pipe */
 			if (pipe(service->status_fd) < 0) {
@@ -519,6 +522,10 @@ void service_monitor_stop(struct service *service)
 			service->status_fd[i] = -1;
 		}
 	}
+	if (service->master_dead_pipe_fd[0] != -1) {
+		i_close_fd(&service->master_dead_pipe_fd[0]);
+		i_close_fd(&service->master_dead_pipe_fd[1]);
+	}
 	if (service->login_notify_fd != -1) {
 		if (close(service->login_notify_fd) < 0) {
 			service_error(service,
@@ -561,15 +568,6 @@ void services_monitor_stop(struct service_list *service_list, bool wait)
 {
 	struct service *const *services;
 
-	if (service_list->master_dead_pipe_fd[0] != -1) {
-		if (close(service_list->master_dead_pipe_fd[0]) < 0)
-			i_error("close(master dead pipe) failed: %m");
-		if (close(service_list->master_dead_pipe_fd[1]) < 0)
-			i_error("close(master dead pipe) failed: %m");
-		service_list->master_dead_pipe_fd[0] = -1;
-		service_list->master_dead_pipe_fd[1] = -1;
-	}
-
 	if (wait) {
 		/* we've notified all children that the master is dead.
 		   now wait for the children to either die or to tell that
diff --git a/src/master/service-process.c b/src/master/service-process.c
index d0f1df6..7a24c58 100644
--- a/src/master/service-process.c
+++ b/src/master/service-process.c
@@ -133,7 +133,7 @@ service_dup_fds(struct service *service)
 	}
 	dup2_append(&dups, service->status_fd[1], MASTER_STATUS_FD);
 	if (service->type != SERVICE_TYPE_ANVIL) {
-		dup2_append(&dups, service->list->master_dead_pipe_fd[1],
+		dup2_append(&dups, service->master_dead_pipe_fd[1],
 			    MASTER_DEAD_FD);
 	} else {
 		dup2_append(&dups, global_master_dead_pipe_fd[1],
diff --git a/src/master/service.c b/src/master/service.c
index d9f49d6..d175b0c 100644
--- a/src/master/service.c
+++ b/src/master/service.c
@@ -283,6 +283,8 @@ service_create(pool_t pool, const struct service_settings *set,
 	service->log_fd[1] = -1;
 	service->status_fd[0] = -1;
 	service->status_fd[1] = -1;
+	service->master_dead_pipe_fd[0] = -1;
+	service->master_dead_pipe_fd[1] = -1;
 	service->log_process_internal_fd = -1;
 	service->login_notify_fd = -1;
 
@@ -424,8 +426,6 @@ services_create_real(const struct master_settings *set, pool_t pool,
 	service_list->set = set;
 	service_list->master_log_fd[0] = -1;
 	service_list->master_log_fd[1] = -1;
-	service_list->master_dead_pipe_fd[0] = -1;
-	service_list->master_dead_pipe_fd[1] = -1;
 
 	service_settings = array_get(&set->services, &count);
 	p_array_init(&service_list->services, pool, count);
diff --git a/src/master/service.h b/src/master/service.h
index 79aa65f..3887e52 100644
--- a/src/master/service.h
+++ b/src/master/service.h
@@ -85,6 +85,8 @@ struct service {
 	int status_fd[2];
 	struct io *io_status;
 
+	int master_dead_pipe_fd[2];
+
 	unsigned int throttle_secs;
 	time_t exit_failure_last;
 	unsigned int exit_failures_in_sec;
@@ -139,8 +141,6 @@ struct service_list {
 	int master_log_fd[2];
 	struct service_process_notify *log_byes;
 
-	int master_dead_pipe_fd[2];
-
 	ARRAY(struct service *) services;
 
 	unsigned int destroying:1;
